"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
/**
 * Tencent is pleased to support the open source community by making CloudBaseFramework - 云原生一体化部署工具 available.
 *
 * Copyright (C) 2020 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * Please refer to license text included with this package for license details.
 */
const framework_core_1 = require("@cloudbase/framework-core");
const change_case_1 = require("change-case");
class DatabasePlugin extends framework_core_1.Plugin {
    constructor(name, api, inputs) {
        super(name, api, inputs);
        this.name = name;
        this.api = api;
        this.inputs = inputs;
        const DEFAULT_INPUTS = {};
        this.resolvedInputs = resolveInputs(this.inputs, DEFAULT_INPUTS);
    }
    /**
     * 初始化
     */
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('DatabasePlugin: init', this.resolvedInputs);
        });
    }
    /**
     * 执行本地命令
     */
    run() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * 删除资源
     */
    remove() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * 生成代码
     */
    genCode() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * 构建
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('DatabasePlugin: build', this.resolvedInputs);
        });
    }
    /**
     * 生成SAM文件
     */
    compile() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('DatabasePlugin: compile', this.resolvedInputs);
            return {
                Resources: this.resolvedInputs.collections.reduce((prev, cur) => {
                    prev[change_case_1.constantCase(cur.collectionName)] = this.toSAM(cur);
                    return prev;
                }, {}),
            };
        });
    }
    /**
     * 部署
     */
    deploy() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('DatabasePlugin: deploy', this.resolvedInputs, this.buildOutput);
        });
    }
    toSAM(collectionConfig) {
        let properties = JSON.parse(JSON.stringify(collectionConfig, replacer));
        function replacer(key, value) {
            if (value && typeof value === 'object') {
                let replacement = Array.isArray(value) ? [] : {};
                for (var k in value) {
                    if (Object.hasOwnProperty.call(value, k)) {
                        let v = value[k];
                        if (k === 'aclRule') {
                            v = JSON.stringify(v);
                        }
                        replacement[change_case_1.pascalCase(k)] = v;
                    }
                }
                return replacement;
            }
            return value;
        }
        return {
            Type: 'CloudBase::FlexDB',
            Properties: Object.assign({
                Description: '云开发 NoSQL 数据库',
            }, properties),
        };
    }
    toConstantCase(name) {
        let result = '';
        let lastIsDivide = true;
        for (let letter of name) {
            if (letter === '-' || letter === '_') {
                lastIsDivide = true;
            }
            else if (lastIsDivide) {
                result += letter.toUpperCase();
                lastIsDivide = false;
            }
            else {
                result += letter.toLowerCase();
                lastIsDivide = false;
            }
        }
        return result;
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
exports.plugin = DatabasePlugin;
